@@ -106,20 +106,25 @@ void make_reject_map(            //make rej map for wd //detailed results
                      ROW *row,
                      INT16 pass  //1st or 2nd?
                     );
-void reject_blanks(WERD_RES *word); 
-void reject_I_1_L(WERD_RES *word); 
+void reject_blanks(WERD_RES *word);
+void reject_I_1_L(WERD_RES *word);
                                  //detailed results
-void reject_poor_matches(WERD_RES *word, BLOB_CHOICE_LIST_CLIST *blob_choices); 
+void reject_poor_matches(WERD_RES *word, BLOB_CHOICE_LIST_CLIST *blob_choices);
 float compute_reject_threshold(  //compute threshold //detailed results
                                BLOB_CHOICE_LIST_CLIST *blob_choices);
 int sort_floats(                   //qsort function
                 const void *arg1,  //ptrs to floats
                 const void *arg2);
-void reject_edge_blobs(WERD_RES *word); 
-BOOL8 one_ell_conflict(WERD_RES *word_res, BOOL8 update_map); 
-INT16 first_alphanum_pos(const char *word); 
-INT16 alpha_count(const char *word); 
-BOOL8 word_contains_non_1_digit(const char *word); 
+void reject_edge_blobs(WERD_RES *word);
+BOOL8 one_ell_conflict(WERD_RES *word_res, BOOL8 update_map);
+INT16 first_alphanum_offset(const char *word,
+                            const char *word_lengths);
+INT16 first_alphanum_index(const char *word,
+                           const char *word_lengths);
+INT16 alpha_count(const char *word,
+                  const char *word_lengths);
+BOOL8 word_contains_non_1_digit(const char *word,
+                                const char *word_lengths);
 BOOL8 test_ambig_word(  //test for ambiguity
                       WERD_RES *word);
                                  //original word
@@ -127,13 +132,13 @@ BOOL8 ambig_word(const char *start_word,
                  char *temp_word,     //alterable copy
                  INT16 test_char_pos  //idx to char to alter
                 );
-const char *char_ambiguities(char c); 
+const char *char_ambiguities(char c);
 
 #ifndef EMBEDDED
 void test_ambigs(const char *word);
 #endif
- 
-void nn_recover_rejects(WERD_RES *word, ROW *row); 
+
+void nn_recover_rejects(WERD_RES *word, ROW *row);
 void nn_match_word(  //Match a word
                    WERD_RES *word,
                    ROW *row);
@@ -157,19 +162,20 @@ INT16 evaluate_net_match(char top,
                          BOOL8 sensible_word,
                          BOOL8 centre,
                          BOOL8 good_quality_word);
-void dont_allow_dubious_chars(WERD_RES *word); 
+void dont_allow_dubious_chars(WERD_RES *word);
 
-void dont_allow_1Il(WERD_RES *word); 
+void dont_allow_1Il(WERD_RES *word);
 
 INT16 count_alphanums(  //how many alphanums
                       WERD_RES *word);
 void reject_mostly_rejects(  //rej all if most rejectd
                            WERD_RES *word);
-BOOL8 repeated_nonalphanum_wd(WERD_RES *word, ROW *row); 
-BOOL8 repeated_ch_string(const char *rep_ch_str); 
-INT16 safe_dict_word(const char *s); 
-void flip_hyphens(WERD_RES *word); 
-void flip_0O(WERD_RES *word); 
-BOOL8 non_O_upper(char c); 
-BOOL8 non_0_digit(char c); 
+BOOL8 repeated_nonalphanum_wd(WERD_RES *word, ROW *row);
+BOOL8 repeated_ch_string(const char *rep_ch_str,
+                         const char *lengths);
+INT16 safe_dict_word(const char *s);
+void flip_hyphens(WERD_RES *word);
+void flip_0O(WERD_RES *word);
+BOOL8 non_O_upper(const char* str, int length);
+BOOL8 non_0_digit(const char* str, int length);
 #endif
