@@ -4,7 +4,11 @@
 #define SIGNED
 #else
 #define __UNIX__
-#include <linux/limits.h>
+#include <limits.h>
+#ifndef PATH_MAX
+#define MAX_PATH 4096
+#else
 #define MAX_PATH PATH_MAX
+#endif
 #define SIGNED signed
 #endif
