@@ -22,6 +22,7 @@
 #include "adaptive.h"
 #include "emalloc.h"
 #include "freelist.h"
+#include "globals.h"
 
 #ifdef __UNIX__
 #include <assert.h>
@@ -69,7 +70,7 @@ int AddAdaptedClass(ADAPT_TEMPLATES Templates,
 
 
 /*---------------------------------------------------------------------------*/
-void FreeTempConfig(TEMP_CONFIG Config) { 
+void FreeTempConfig(TEMP_CONFIG Config) {
 /*
  **	Parameters:
  **		Config	config to be freed
@@ -90,7 +91,7 @@ void FreeTempConfig(TEMP_CONFIG Config) {
 
 
 /*---------------------------------------------------------------------------*/
-void FreeTempProto(void *arg) { 
+void FreeTempProto(void *arg) {
   PROTO proto = (PROTO) arg;
 
   c_free_struct (proto, sizeof (TEMP_PROTO_STRUCT), "TEMP_PROTO_STRUCT");
@@ -98,7 +99,7 @@ void FreeTempProto(void *arg) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_CLASS NewAdaptedClass() { 
+ADAPT_CLASS NewAdaptedClass() {
 /*
  **	Parameters: none
  **	Globals: none
@@ -129,7 +130,7 @@ ADAPT_CLASS NewAdaptedClass() {
 
 
 /*-------------------------------------------------------------------------*/
-void free_adapted_class(ADAPT_CLASS adapt_class) { 
+void free_adapted_class(ADAPT_CLASS adapt_class) {
   int i;
 
   for (i = 0; i < MAX_NUM_CONFIGS; i++) {
@@ -143,12 +144,12 @@ void free_adapted_class(ADAPT_CLASS adapt_class) {
   FreeBitVector (adapt_class->PermProtos);
   FreeBitVector (adapt_class->PermConfigs);
   destroy_nodes (adapt_class->TempProtos, FreeTempProto);
-  Efree(adapt_class); 
+  Efree(adapt_class);
 }
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_TEMPLATES NewAdaptedTemplates() { 
+ADAPT_TEMPLATES NewAdaptedTemplates() {
 /*
  **	Parameters: none
  **	Globals: none
@@ -174,20 +175,20 @@ ADAPT_TEMPLATES NewAdaptedTemplates() {
 
 
 /*-------------------------------------------------------------------------------*/
-void free_adapted_templates(ADAPT_TEMPLATES templates) { 
+void free_adapted_templates(ADAPT_TEMPLATES templates) {
 
   if (templates != NULL) {
     int i;
     for (i = 0; i < NumClassesIn (templates->Templates); i++)
       free_adapted_class (templates->Class[i]);
     free_int_templates (templates->Templates);
-    Efree(templates); 
+    Efree(templates);
   }
 }
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_CONFIG NewTempConfig(int MaxProtoId) { 
+TEMP_CONFIG NewTempConfig(int MaxProtoId) {
 /*
  **	Parameters:
  **		MaxProtoId	max id of any proto in new config
@@ -218,7 +219,7 @@ TEMP_CONFIG NewTempConfig(int MaxProtoId) {
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_PROTO NewTempProto() { 
+TEMP_PROTO NewTempProto() {
 /*
  **	Parameters: none
  **	Globals: none
@@ -233,7 +234,7 @@ TEMP_PROTO NewTempProto() {
 
 
 /*---------------------------------------------------------------------------*/
-void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) { 
+void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
 /*
  **	Parameters:
  **		File		open text file to print Templates to
@@ -260,8 +261,8 @@ void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
     IClass = ClassForIndex (Templates->Templates, i);
     AClass = Templates->Class[i];
 
-    fprintf (File, "%5d  %c %3d %3d %3d %3d\n",
-      i, ClassIdForIndex (Templates->Templates, i),
+    fprintf (File, "%5d  %s %3d %3d %3d %3d\n",
+      i, unicharset.id_to_unichar(ClassIdForIndex (Templates->Templates, i)),
       NumIntConfigsIn (IClass), AClass->NumPermConfigs,
       NumIntProtosIn (IClass),
       NumIntProtosIn (IClass) - count (AClass->TempProtos));
@@ -273,7 +274,7 @@ void PrintAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_CLASS ReadAdaptedClass(FILE *File) { 
+ADAPT_CLASS ReadAdaptedClass(FILE *File) {
 /*
  **	Parameters:
  **		File	open file to read adapted class from
@@ -327,7 +328,7 @@ ADAPT_CLASS ReadAdaptedClass(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) { 
+ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) {
 /*
  **	Parameters:
  **		File	open text file to read adapted templates from
@@ -358,7 +359,7 @@ ADAPT_TEMPLATES ReadAdaptedTemplates(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-PERM_CONFIG ReadPermConfig(FILE *File) { 
+PERM_CONFIG ReadPermConfig(FILE *File) {
 /*
  **	Parameters:
  **		File	open file to read permanent config from
@@ -374,8 +375,8 @@ PERM_CONFIG ReadPermConfig(FILE *File) {
 
   fread ((char *) &NumAmbigs, sizeof (UINT8), 1, File);
   Config = (PERM_CONFIG) Emalloc (sizeof (char) * (NumAmbigs + 1));
-  fread (Config, sizeof (char), NumAmbigs, File);
-  Config[NumAmbigs] = '\0';
+  fread (Config, sizeof (UNICHAR_ID), NumAmbigs, File);
+  Config[NumAmbigs] = -1;
 
   return (Config);
 
@@ -383,7 +384,7 @@ PERM_CONFIG ReadPermConfig(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-TEMP_CONFIG ReadTempConfig(FILE *File) { 
+TEMP_CONFIG ReadTempConfig(FILE *File) {
 /*
  **	Parameters:
  **		File	open file to read temporary config from
@@ -411,7 +412,7 @@ TEMP_CONFIG ReadTempConfig(FILE *File) {
 
 
 /*---------------------------------------------------------------------------*/
-void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) { 
+void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
 /*
  **	Parameters:
  **		File		open file to write Class to
@@ -442,7 +443,7 @@ void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
   fwrite ((char *) &NumTempProtos, sizeof (int), 1, File);
   TempProtos = Class->TempProtos;
   iterate (TempProtos) {
-    void* proto = first(TempProtos);
+    void* proto = first_node(TempProtos);
     fwrite ((char *) proto, sizeof (TEMP_PROTO_STRUCT), 1, File);
   }
 
@@ -458,7 +459,7 @@ void WriteAdaptedClass(FILE *File, ADAPT_CLASS Class, int NumConfigs) {
 
 
 /*---------------------------------------------------------------------------*/
-void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) { 
+void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
 /*
  **	Parameters:
  **		File		open text file to write Templates to
@@ -475,7 +476,7 @@ void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
   fwrite ((char *) Templates, sizeof (ADAPT_TEMPLATES_STRUCT), 1, File);
 
   /* then write out the basic integer templates */
-  WriteIntTemplates (File, Templates->Templates);
+  WriteIntTemplates (File, Templates->Templates, unicharset);
 
   /* then write out the adaptive info for each class */
   for (i = 0; i < NumClassesIn (Templates->Templates); i++) {
@@ -487,7 +488,7 @@ void WriteAdaptedTemplates(FILE *File, ADAPT_TEMPLATES Templates) {
 
 
 /*---------------------------------------------------------------------------*/
-void WritePermConfig(FILE *File, PERM_CONFIG Config) { 
+void WritePermConfig(FILE *File, PERM_CONFIG Config) {
 /*
  **	Parameters:
  **		File	open file to write Config to
@@ -499,19 +500,20 @@ void WritePermConfig(FILE *File, PERM_CONFIG Config) {
  **	Exceptions: none
  **	History: Tue Mar 19 13:55:44 1991, DSJ, Created.
  */
-  UINT8 NumAmbigs;
+  UINT8 NumAmbigs = 0;
 
   assert (Config != NULL);
+  while (Config[NumAmbigs] > 0)
+    ++NumAmbigs;
 
-  NumAmbigs = strlen (Config);
   fwrite ((char *) &NumAmbigs, sizeof (UINT8), 1, File);
-  fwrite (Config, sizeof (char), NumAmbigs, File);
+  fwrite (Config, sizeof (UNICHAR_ID), NumAmbigs, File);
 
 }                                /* WritePermConfig */
 
 
 /*---------------------------------------------------------------------------*/
-void WriteTempConfig(FILE *File, TEMP_CONFIG Config) { 
+void WriteTempConfig(FILE *File, TEMP_CONFIG Config) {
 /*
  **	Parameters:
  **		File	open file to write Config to
