@@ -22,6 +22,7 @@
 #include "intmatcher.h"
 #include "tordvars.h"
 #include "callcpp.h"
+#include "globals.h"
 #include <math.h>
 
 #define CLASS_MASK_SIZE ((MAX_NUM_CLASSES*NUM_BITS_PER_CLASS \
@@ -323,11 +324,11 @@ make_int_var (ClassPrunerThreshold, 229, MakeClassPrunerThreshold,
 16, 20, SetClassPrunerThreshold,
 "Class Pruner Threshold 0-255:        ");
 
-make_int_var (ClassPrunerMultiplier, 15, MakeClassPrunerMultiplier,
+make_int_var (ClassPrunerMultiplier, 30, MakeClassPrunerMultiplier,
 16, 21, SetClassPrunerMultiplier,
 "Class Pruner Multiplier 0-255:       ");
 
-make_int_var (IntegerMatcherMultiplier, 7, MakeIntegerMatcherMultiplier,
+make_int_var (IntegerMatcherMultiplier, 14, MakeIntegerMatcherMultiplier,
 16, 22, SetIntegerMatcherMultiplier,
 "Integer Matcher Multiplier  0-255:   ");
 
@@ -335,7 +336,7 @@ make_int_var (IntThetaFudge, 128, MakeIntThetaFudge,
 16, 23, SetIntThetaFudge,
 "Integer Matcher Theta Fudge 0-255:   ");
 
-make_float_var (CPCutoffStrength, 0.15, MakeCPCutoffStrength,
+make_int_var (CPCutoffStrength, 7, MakeCPCutoffStrength,
 16, 24, SetCPCutoffStrength,
 "Class Pruner CutoffStrength:         ");
 
@@ -422,10 +423,10 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
   int NumPruners;
   INT32 feature_index;           //current feature
 
-  static INT32 ClassCount[MAX_NUM_CLASSES - 1];
-  static INT16 NormCount[MAX_NUM_CLASSES - 1];
-  static INT16 SortKey[MAX_NUM_CLASSES];
-  static UINT8 SortIndex[MAX_NUM_CLASSES];
+  static INT32 ClassCount[MAX_NUM_CLASSES];
+  static INT16 NormCount[MAX_NUM_CLASSES];
+  static INT16 SortKey[MAX_NUM_CLASSES + 1];
+  static UINT8 SortIndex[MAX_NUM_CLASSES + 1];
   CLASS_INDEX Class;
   int out_class;
   int MaxNumClasses;
@@ -433,7 +434,7 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
   int NumClasses;
   FLOAT32 max_rating;            //max allowed rating
   INT32 *ClassCountPtr;
-  INT8 classch;
+  CLASS_ID classch;
 
   MaxNumClasses = NumClassesIn (IntTemplates);
 
@@ -497,12 +498,11 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
 
   /* Adjust Class Counts for Number of Expected Features */
   for (Class = 0; Class < MaxNumClasses; Class++)
-    if (NumFeatures < ExpectedNumFeatures[Class])
-      ClassCount[Class] =
-        (int) (((FLOAT32) (ClassCount[Class] * NumFeatures)) /
-        (NumFeatures +
-        CPCutoffStrength * (ExpectedNumFeatures[Class] -
-        NumFeatures)));
+    if (NumFeatures < ExpectedNumFeatures[Class]) {
+      int deficit = ExpectedNumFeatures[Class] - NumFeatures;
+      ClassCount[Class] -= ClassCount[Class] * deficit /
+                           (NumFeatures*CPCutoffStrength + deficit);
+    }
 
   /* Adjust Class Counts for Normalization Factors */
   MaxCount = 0;
@@ -530,22 +530,19 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
 
   /* Sort Classes using Heapsort Algorithm */
   if (NumClasses > 1)
-    HeapSort(NumClasses, SortKey, SortIndex); 
+    HeapSort(NumClasses, SortKey, SortIndex);
 
   if (display_ratings > 1) {
     cprintf ("CP:%d classes, %d features:\n", NumClasses, NumFeatures);
     for (Class = 0; Class < NumClasses; Class++) {
-      classch =
-        ClassIdForIndex (IntTemplates, SortIndex[NumClasses - Class]);
-      cprintf ("%c:C=%d, E=%d, N=%d, Rat=%d\n", classch,
-        ClassCount[SortIndex[NumClasses - Class]],
-        ExpectedNumFeatures[SortIndex[NumClasses - Class]],
-        SortKey[NumClasses - Class],
-        (int) (10 +
-        1000 * (1.0f -
-        SortKey[NumClasses -
-        Class] / ((float) cp_maps[3] *
-        NumFeatures))));
+      classch = ClassIdForIndex (IntTemplates, SortIndex[NumClasses - Class]);
+      cprintf ("%s:C=%d, E=%d, N=%d, Rat=%d\n",
+               unicharset.id_to_unichar(classch),
+               ClassCount[SortIndex[NumClasses - Class]],
+               ExpectedNumFeatures[SortIndex[NumClasses - Class]],
+               SortKey[NumClasses - Class],
+               1010 - 1000 * SortKey[NumClasses - Class] /
+                 (cp_maps[3] * NumFeatures));
     }
     if (display_ratings > 2) {
       NumPruners = NumClassPrunersIn (IntTemplates);
@@ -569,9 +566,9 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
             PrunerWord = *BasePrunerAddress++;
             for (Class = 0; Class < 16; Class++, class_index++) {
               if (NormCount[class_index] >= MaxCount)
-                cprintf (" %c=%d,",
-                  ClassIdForIndex (IntTemplates,
-                  class_index),
+                cprintf (" %s=%d,",
+                  unicharset.id_to_unichar(ClassIdForIndex (IntTemplates,
+                                                            class_index)),
                   PrunerWord & 3);
               PrunerWord >>= 2;
             }
@@ -582,8 +579,8 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
       cprintf ("Adjustments:");
       for (Class = 0; Class < MaxNumClasses; Class++) {
         if (NormCount[Class] > MaxCount)
-          cprintf (" %c=%d,",
-            ClassIdForIndex (IntTemplates, Class),
+          cprintf (" %s=%d,",
+            unicharset.id_to_unichar(ClassIdForIndex (IntTemplates, Class)),
             -((ClassPrunerMultiplier *
             NormalizationFactors[Class]) >> 8) * cp_maps[3] /
             3);
@@ -640,7 +637,7 @@ int ClassPruner(INT_TEMPLATES IntTemplates,
     cp_bestconf = -1;
     for (Class = 0; Class < NumClasses; Class++) {
       classch = Results[Class].Class;
-      if (classch == blob_answer) {
+      if (strcmp(unicharset.id_to_unichar(classch), blob_answer) == 0) {
         cp_bestindex = Class;
         cp_bestrating = (int) (1000 * Results[Class].Rating + 10);
         cp_bestconf = (int) (1000 * Results[Class].Rating2 + 10);
@@ -947,8 +944,10 @@ void PruningMatcher(INT_CLASS ClassTemplate,
                     NormalizationFactor,
                     Result);
 
+#ifndef GRAPHICS_DISABLED
   if (PrintMatchSummaryOn (Debug))
-    IMDebugBestMatch(BestMatch, Result, BlobLength, NormalizationFactor); 
+    IMDebugBestMatch(BestMatch, Result, BlobLength, NormalizationFactor);
+#endif
 
   if (MatchDebuggingOn (Debug))
     cprintf ("Match Complete --------------------------------------------\n");
@@ -1032,7 +1031,7 @@ void IntegerMatcher(INT_CLASS ClassTemplate,
   if (MatchDebuggingOn (Debug))
     cprintf ("Integer Matcher -------------------------------------------\n");
 
-  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence); 
+  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence);
 
   for (Feature = 0, used_features = 0; Feature < NumFeatures; Feature++) {
     if (Features[Feature].CP_misses >= min_misses) {
@@ -1044,6 +1043,7 @@ void IntegerMatcher(INT_CLASS ClassTemplate,
     }
   }
 
+#ifndef GRAPHICS_DISABLED
   if (PrintProtoMatchesOn (Debug) || PrintMatchSummaryOn (Debug))
     IMDebugFeatureProtoError(ClassTemplate,
                              ProtoMask,
@@ -1067,6 +1067,7 @@ void IntegerMatcher(INT_CLASS ClassTemplate,
                               NumFeatures,
                               Features,
                               Debug);
+#endif
 
   IMUpdateSumOfProtoEvidences(ClassTemplate,
                               ConfigMask,
@@ -1086,11 +1087,13 @@ void IntegerMatcher(INT_CLASS ClassTemplate,
                     NormalizationFactor,
                     Result);
 
+#ifndef GRAPHICS_DISABLED
   if (PrintMatchSummaryOn (Debug))
-    IMDebugBestMatch(BestMatch, Result, BlobLength, NormalizationFactor); 
+    IMDebugBestMatch(BestMatch, Result, BlobLength, NormalizationFactor);
 
   if (MatchDebuggingOn (Debug))
     cprintf ("Match Complete --------------------------------------------\n");
+#endif
 
 }
 
@@ -1144,13 +1147,14 @@ int FindGoodProtos(INT_CLASS ClassTemplate,
     cprintf
       ("Find Good Protos -------------------------------------------\n");
 
-  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence); 
+  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence);
 
   for (Feature = 0; Feature < NumFeatures; Feature++)
     IMUpdateTablesForFeature (ClassTemplate, ProtoMask, ConfigMask, Feature,
       &(Features[Feature]), FeatureEvidence,
       SumOfFeatureEvidence, ProtoEvidence, Debug);
 
+#ifndef GRAPHICS_DISABLED
   if (PrintProtoMatchesOn (Debug) || PrintMatchSummaryOn (Debug))
     IMDebugFeatureProtoError(ClassTemplate,
                              ProtoMask,
@@ -1159,6 +1163,7 @@ int FindGoodProtos(INT_CLASS ClassTemplate,
                              ProtoEvidence,
                              NumFeatures,
                              Debug);
+#endif
 
   /* Average Proto Evidences & Find Good Protos */
   NumProtos = NumIntProtosIn (ClassTemplate);
@@ -1183,7 +1188,6 @@ int FindGoodProtos(INT_CLASS ClassTemplate,
 
   if (MatchDebuggingOn (Debug))
     cprintf ("Match Complete --------------------------------------------\n");
-
   return NumGoodProtos;
 
 }
@@ -1237,7 +1241,7 @@ int FindBadFeatures(INT_CLASS ClassTemplate,
     cprintf
       ("Find Bad Features -------------------------------------------\n");
 
-  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence); 
+  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence);
 
   NumBadFeatures = 0;
   NumConfigs = NumIntConfigsIn (ClassTemplate);
@@ -1261,6 +1265,7 @@ int FindBadFeatures(INT_CLASS ClassTemplate,
     }
   }
 
+#ifndef GRAPHICS_DISABLED
   if (PrintProtoMatchesOn (Debug) || PrintMatchSummaryOn (Debug))
     IMDebugFeatureProtoError(ClassTemplate,
                              ProtoMask,
@@ -1269,6 +1274,7 @@ int FindBadFeatures(INT_CLASS ClassTemplate,
                              ProtoEvidence,
                              NumFeatures,
                              Debug);
+#endif
 
   if (MatchDebuggingOn (Debug))
     cprintf ("Match Complete --------------------------------------------\n");
@@ -1279,7 +1285,7 @@ int FindBadFeatures(INT_CLASS ClassTemplate,
 
 
 /*---------------------------------------------------------------------------*/
-void InitIntegerMatcher() { 
+void InitIntegerMatcher() {
   int i;
   UINT32 IntSimilarity;
   double Similarity;
@@ -1287,7 +1293,7 @@ void InitIntegerMatcher() {
   double ScaleFactor;
 
   /* Set default mode of operation of IntegerMatcher */
-  SetCharNormMatch(); 
+  SetCharNormMatch();
 
   /* Initialize table for evidence to similarity lookup */
   for (i = 0; i < SE_TABLE_SIZE; i++) {
@@ -1323,21 +1329,21 @@ void InitIntegerMatcher() {
 
 
 /*---------------------------------------------------------------------------*/
-void InitIntegerMatcherVars() { 
-  MakeClassPrunerThreshold(); 
-  MakeClassPrunerMultiplier(); 
-  MakeIntegerMatcherMultiplier(); 
-  MakeIntThetaFudge(); 
-  MakeCPCutoffStrength(); 
-  MakeEvidenceTableBits(); 
-  MakeIntEvidenceTruncBits(); 
-  MakeSEExponentialMultiplier(); 
-  MakeSimilarityCenter(); 
+void InitIntegerMatcherVars() {
+  MakeClassPrunerThreshold();
+  MakeClassPrunerMultiplier();
+  MakeIntegerMatcherMultiplier();
+  MakeIntThetaFudge();
+  MakeCPCutoffStrength();
+  MakeEvidenceTableBits();
+  MakeIntEvidenceTruncBits();
+  MakeSEExponentialMultiplier();
+  MakeSimilarityCenter();
 }
 
 
 /*-------------------------------------------------------------------------*/
-void PrintIntMatcherStats(FILE *f) { 
+void PrintIntMatcherStats(FILE *f) {
   fprintf (f, "protoword_lookups=%d, zero_protowords=%d, proto_shifts=%d\n",
     protoword_lookups, zero_protowords, proto_shifts);
   fprintf (f, "set_proto_bits=%d, config_shifts=%d, set_config_bits=%d\n",
@@ -1346,7 +1352,7 @@ void PrintIntMatcherStats(FILE *f) {
 
 
 /*-------------------------------------------------------------------------*/
-void SetProtoThresh(FLOAT32 Threshold) { 
+void SetProtoThresh(FLOAT32 Threshold) {
   AdaptProtoThresh = (int) (255 * Threshold);
   if (AdaptProtoThresh < 0)
     AdaptProtoThresh = 0;
@@ -1356,7 +1362,7 @@ void SetProtoThresh(FLOAT32 Threshold) {
 
 
 /*---------------------------------------------------------------------------*/
-void SetFeatureThresh(FLOAT32 Threshold) { 
+void SetFeatureThresh(FLOAT32 Threshold) {
   AdaptFeatureThresh = (int) (255 * Threshold);
   if (AdaptFeatureThresh < 0)
     AdaptFeatureThresh = 0;
@@ -1366,13 +1372,13 @@ void SetFeatureThresh(FLOAT32 Threshold) {
 
 
 /*--------------------------------------------------------------------------*/
-void SetBaseLineMatch() { 
+void SetBaseLineMatch() {
   LocalMatcherMultiplier = 0;
 }
 
 
 /*--------------------------------------------------------------------------*/
-void SetCharNormMatch() { 
+void SetCharNormMatch() {
   LocalMatcherMultiplier = IntegerMatcherMultiplier;
 }
 
@@ -1804,6 +1810,7 @@ int Debug) {
 
 
 /*---------------------------------------------------------------------------*/
+#ifndef GRAPHICS_DISABLED
 void
 IMDebugFeatureProtoError (INT_CLASS ClassTemplate,
 BIT_VECTOR ProtoMask,
@@ -2023,7 +2030,7 @@ void IMDisplayFeatureDebugInfo(INT_CLASS ClassTemplate,
   int NumConfigs;
   register int Temp;
 
-  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence); 
+  IMClearTables(ClassTemplate, SumOfFeatureEvidence, ProtoEvidence);
 
   NumConfigs = NumIntConfigsIn (ClassTemplate);
   for (Feature = 0; Feature < NumFeatures; Feature++) {
@@ -2050,7 +2057,7 @@ void IMDisplayFeatureDebugInfo(INT_CLASS ClassTemplate,
     }
   }
 }
-
+#endif
 
 /*---------------------------------------------------------------------------*/
 void
@@ -2226,6 +2233,7 @@ UINT8 NormalizationFactor, INT_RESULT Result) {
 
 
 /*---------------------------------------------------------------------------*/
+#ifndef GRAPHICS_DISABLED
 void IMDebugBestMatch(int BestMatch,
                       INT_RESULT Result,
                       UINT16 BlobLength,
@@ -2250,7 +2258,7 @@ void IMDebugBestMatch(int BestMatch,
     100.0 * NormalizationFactor / 256.0, LocalMatcherMultiplier,
     100.0 * LocalMatcherMultiplier / (BlobLength + LocalMatcherMultiplier));
 }
-
+#endif
 
 /*---------------------------------------------------------------------------*/
 void
