@@ -54,10 +54,12 @@ void append_next_choice(EDGE_ARRAY dawg,
                         NODE_REF node,
                         char permuter,
                         char *word,
+                        char unichar_lengths[],
+                        int unichar_offsets[],
                         CHOICES_LIST choices,
                         int char_index,
                         A_CHOICE *this_choice,
-                        char prevchar,
+                        const char *prevchar,
                         float *limit,
                         float rating,
                         float certainty,
@@ -74,6 +76,8 @@ CHOICES dawg_permute(EDGE_ARRAY dawg,
                      int char_index,
                      float *limit,
                      char *word,
+                     char unichar_lengths[],
+                     int unichar_offsets[],
                      float rating,
                      float certainty,
                      float *rating_array,
