@@ -31,6 +31,8 @@
 #include "freelist.h"
 #include "efio.h"
 #include "globals.h"
+#include "scanutils.h"
+#include "unichar.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -45,13 +47,13 @@ extern float CertaintyScale;     /* from subfeat.h */
 
 #define MAX_WERD_SIZE   100
 #define MAX_AMBIG_SIZE    3
-#define DANGEROUS_AMBIGS  "tessdata/DangAmbigs"
+#define DANGEROUS_AMBIGS  "DangAmbigs"
 
 typedef LIST AMBIG_TABLE;
 
 typedef struct
 {
-  UINT8 Class;
+  UNICHAR_ID Class;
   UINT16 NumChunks;
   float Certainty;
 }
@@ -73,18 +75,24 @@ typedef struct
 {
   VIABLE_CHOICE Choice;
   float ChunkCertainty[MAX_NUM_CHUNKS];
-  UINT8 ChunkClass[MAX_NUM_CHUNKS];
+  UNICHAR_ID ChunkClass[MAX_NUM_CHUNKS];
 }
 
 
 EXPANDED_CHOICE;
 
+typedef struct
+{
+  char ambig[2 * (UNICHAR_LEN * MAX_AMBIG_SIZE) + 2];
+  char lengths[2 * (MAX_AMBIG_SIZE) + 2];
+} AMBIG_SPEC;
+
 /**----------------------------------------------------------------------------
           Macros
 ----------------------------------------------------------------------------**/
-#define BestCertainty(Choices)  (((VIABLE_CHOICE) first (Choices))->Certainty)
-#define BestRating(Choices) (((VIABLE_CHOICE) first (Choices))->Rating)
-#define BestFactor(Choices) (((VIABLE_CHOICE) first (Choices))->AdjustFactor)
+#define BestCertainty(Choices)  (((VIABLE_CHOICE) first_node (Choices))->Certainty)
+#define BestRating(Choices) (((VIABLE_CHOICE) first_node (Choices))->Rating)
+#define BestFactor(Choices) (((VIABLE_CHOICE) first_node (Choices))->AdjustFactor)
 
 #define AmbigThreshold(F1,F2)	(((F2) - (F1)) * AmbigThresholdGain - \
 				AmbigThresholdOffset)
@@ -92,40 +100,43 @@ EXPANDED_CHOICE;
 /*---------------------------------------------------------------------------
           Private Function Prototoypes
 ----------------------------------------------------------------------------*/
-void AddNewChunk(VIABLE_CHOICE Choice, int Blob); 
+void AddNewChunk(VIABLE_CHOICE Choice, int Blob);
 
 int AmbigsFound(char *Word,
                 char *CurrentChar,
-                char *Tail,
+                const char *Tail,
+                const char *Tail_lengths,
                 LIST Ambigs,
                 DANGERR *fixpt);
 
-int ChoiceSameAs(A_CHOICE *Choice, VIABLE_CHOICE ViableChoice); 
+int ChoiceSameAs(A_CHOICE *Choice, VIABLE_CHOICE ViableChoice);
 
 int CmpChoiceRatings(void *arg1,   //VIABLE_CHOICE         Choice1,
                      void *arg2);  //VIABLE_CHOICE         Choice2);
 
-void ExpandChoice(VIABLE_CHOICE Choice, EXPANDED_CHOICE *ExpandedChoice); 
+void ExpandChoice(VIABLE_CHOICE Choice, EXPANDED_CHOICE *ExpandedChoice);
 
-AMBIG_TABLE *FillAmbigTable(); 
+AMBIG_TABLE *FillAmbigTable();
 
 int FreeBadChoice(void *item1,   //VIABLE_CHOICE                 Choice,
                   void *item2);  //EXPANDED_CHOICE                       *BestChoice);
 
-int LengthOfShortestAlphaRun(register char *Word); 
+int LengthOfShortestAlphaRun(register char *Word, const char *Word_lengths);
 
 VIABLE_CHOICE NewViableChoice (A_CHOICE * Choice,
 FLOAT32 AdjustFactor, float Certainties[]);
 
-void PrintViableChoice(FILE *File, const char *Label, VIABLE_CHOICE Choice); 
+void PrintViableChoice(FILE *File, const char *Label, VIABLE_CHOICE Choice);
 
 void ReplaceDuplicateChoice (VIABLE_CHOICE OldChoice,
 A_CHOICE * NewChoice,
 FLOAT32 AdjustFactor, float Certainties[]);
 
-int StringSameAs(const char *String, VIABLE_CHOICE ViableChoice); 
+int StringSameAs(const char *String,
+                 const char *String_lengths,
+                 VIABLE_CHOICE ViableChoice);
 
-int UniformCertainties(CHOICES_LIST Choices, A_CHOICE *BestChoice); 
+int UniformCertainties(CHOICES_LIST Choices, A_CHOICE *BestChoice);
 
 /**----------------------------------------------------------------------------
         Global Data Definitions and Declarations
@@ -135,6 +146,7 @@ static const char *DangerousAmbigs = DANGEROUS_AMBIGS;
 
 /* Word for which stopper debug information should be printed to stdout */
 static char *WordToDebug = NULL;
+static char *WordToDebug_lengths = NULL;
 
 /* flag used to disable accumulation of word choices during compound word
   permutation */
@@ -178,7 +190,6 @@ make_float_var (AmbigThresholdOffset, 1.5, MakeAmbigThresholdOffset,
 17, 9, SetAmbigThresholdOffset,
 "Certainty offset for ambiguity threshold");
 
-//extern char *demodir;
 extern int first_pass;
 INT_VAR (tessedit_truncate_wordchoice_log, 10, "Max words to keep in list");
 
@@ -217,20 +228,25 @@ int AcceptableChoice(CHOICES_LIST Choices,
     cprintf ("\nStopper:  %s (word=%c, case=%c, punct=%c)\n",
       class_string (BestChoice),
       (valid_word (class_string (BestChoice)) ? 'y' : 'n'),
-    (case_ok (class_string (BestChoice)) ? 'y' : 'n'),
-    ((punctuation_ok (class_string (BestChoice)) !=
+    (case_ok (class_string (BestChoice),
+              class_lengths (BestChoice)) ? 'y' : 'n'),
+    ((punctuation_ok (class_string (BestChoice),
+                      class_lengths (BestChoice)) !=
     -1) ? 'y' : 'n'));
 
   if (valid_word (class_string (BestChoice)) &&
-    case_ok (class_string (BestChoice)) &&
-  punctuation_ok (class_string (BestChoice)) != -1) {
-    WordSize = LengthOfShortestAlphaRun (class_string (BestChoice));
+    case_ok (class_string (BestChoice), class_lengths (BestChoice)) &&
+  punctuation_ok (class_string (BestChoice),
+                  class_lengths (BestChoice)) != -1) {
+    WordSize = LengthOfShortestAlphaRun (class_string (BestChoice),
+                                         class_lengths (BestChoice));
     WordSize -= SmallWordSize;
     if (WordSize < 0)
       WordSize = 0;
     CertaintyThreshold += WordSize * CertaintyPerChar;
   }
-  else if (stopper_numbers_on && valid_number (class_string (BestChoice))) {
+  else if (stopper_numbers_on && valid_number (class_string (BestChoice),
+                                               class_lengths (BestChoice))) {
     CertaintyThreshold += stopper_numbers_on * CertaintyPerChar;
   }
 
@@ -238,7 +254,8 @@ int AcceptableChoice(CHOICES_LIST Choices,
     cprintf ("Stopper:  Certainty = %4.1f, Threshold = %4.1f\n",
       class_certainty (BestChoice), CertaintyThreshold);
 
-  if (NoDangerousAmbig (class_string (BestChoice), fixpt)
+  if (NoDangerousAmbig (class_string (BestChoice),
+                        class_lengths (BestChoice), fixpt)
     && class_certainty (BestChoice) > CertaintyThreshold &&
     UniformCertainties (Choices, BestChoice))
     return (TRUE);
@@ -249,7 +266,7 @@ int AcceptableChoice(CHOICES_LIST Choices,
 
 
 /*---------------------------------------------------------------------------*/
-int AcceptableResult(A_CHOICE *BestChoice, A_CHOICE *RawChoice) { 
+int AcceptableResult(A_CHOICE *BestChoice, A_CHOICE *RawChoice) {
 /*
  **	Parameters:
  **		BestChoice	best choice for current word
@@ -274,8 +291,10 @@ int AcceptableResult(A_CHOICE *BestChoice, A_CHOICE *RawChoice) {
     cprintf ("\nRejecter: %s (word=%c, case=%c, punct=%c, unambig=%c)\n",
       class_string (BestChoice),
       (valid_word (class_string (BestChoice)) ? 'y' : 'n'),
-    (case_ok (class_string (BestChoice)) ? 'y' : 'n'),
-    ((punctuation_ok (class_string (BestChoice)) != -1) ? 'y' : 'n'),
+    (case_ok (class_string (BestChoice),
+              class_lengths (BestChoice)) ? 'y' : 'n'),
+    ((punctuation_ok (class_string (BestChoice),
+                      class_lengths (BestChoice)) != -1) ? 'y' : 'n'),
     ((rest (BestChoices) != NIL) ? 'n' : 'y'));
 
   if ((BestChoice == NULL) ||
@@ -283,9 +302,11 @@ int AcceptableResult(A_CHOICE *BestChoice, A_CHOICE *RawChoice) {
     return (FALSE);
 
   if (valid_word (class_string (BestChoice)) &&
-    case_ok (class_string (BestChoice)) &&
-  punctuation_ok (class_string (BestChoice)) != -1) {
-    WordSize = LengthOfShortestAlphaRun (class_string (BestChoice));
+    case_ok (class_string (BestChoice), class_lengths (BestChoice)) &&
+  punctuation_ok (class_string (BestChoice),
+                  class_lengths (BestChoice)) != -1) {
+    WordSize = LengthOfShortestAlphaRun (class_string (BestChoice),
+                                         class_lengths (BestChoice));
     WordSize -= SmallWordSize;
     if (WordSize < 0)
       WordSize = 0;
@@ -310,7 +331,7 @@ int AcceptableResult(A_CHOICE *BestChoice, A_CHOICE *RawChoice) {
 
 
 /*---------------------------------------------------------------------------*/
-int AlternativeChoicesWorseThan(FLOAT32 Threshold) { 
+int AlternativeChoicesWorseThan(FLOAT32 Threshold) {
 /*
  **	Parameters:
  **		Threshold	minimum adjust factor for alternative choices
@@ -327,8 +348,8 @@ int AlternativeChoicesWorseThan(FLOAT32 Threshold) {
   VIABLE_CHOICE Choice;
 
   Alternatives = rest (BestChoices);
-  iterate(Alternatives) { 
-    Choice = (VIABLE_CHOICE) first (Alternatives);
+  iterate(Alternatives) {
+    Choice = (VIABLE_CHOICE) first_node (Alternatives);
     if (Choice->AdjustFactor <= Threshold)
       return (FALSE);
   }
@@ -339,10 +360,11 @@ int AlternativeChoicesWorseThan(FLOAT32 Threshold) {
 
 
 /*---------------------------------------------------------------------------*/
-int CurrentBestChoiceIs(const char *Word) { 
+int CurrentBestChoiceIs(const char *Word, const char *Word_lengths) {
 /*
  **	Parameters:
- **		Word	string to compare to current best choice
+ **		Word            string to compare to current best choice
+ **		Word_lengths	lengths of unichars in Word
  **	Globals:
  **		BestChoices	set of best choices for current word
  **	Operation: Returns TRUE if Word is the same as the current best
@@ -352,13 +374,14 @@ int CurrentBestChoiceIs(const char *Word) {
  **	History: Thu May 30 14:44:22 1991, DSJ, Created.
  */
   return (BestChoices != NIL &&
-    StringSameAs (Word, (VIABLE_CHOICE) first (BestChoices)));
+    StringSameAs (Word, Word_lengths,
+                  (VIABLE_CHOICE) first_node (BestChoices)));
 
 }                                /* CurrentBestChoiceIs */
 
 
 /*---------------------------------------------------------------------------*/
-FLOAT32 CurrentBestChoiceAdjustFactor() { 
+FLOAT32 CurrentBestChoiceAdjustFactor() {
 /*
  **	Parameters: none
  **	Globals:
@@ -374,14 +397,14 @@ FLOAT32 CurrentBestChoiceAdjustFactor() {
   if (BestChoices == NIL)
     return (MAX_FLOAT32);
 
-  BestChoice = (VIABLE_CHOICE) first (BestChoices);
+  BestChoice = (VIABLE_CHOICE) first_node (BestChoices);
   return (BestChoice->AdjustFactor);
 
 }                                /* CurrentBestChoiceAdjustFactor */
 
 
 /*---------------------------------------------------------------------------*/
-int CurrentWordAmbig() { 
+int CurrentWordAmbig() {
 /*
  **	Parameters: none
  **	Globals:
@@ -398,7 +421,7 @@ int CurrentWordAmbig() {
 
 
 /*---------------------------------------------------------------------------*/
-void DebugWordChoices() { 
+void DebugWordChoices() {
 /*
  **	Parameters: none
  **	Globals:
@@ -415,18 +438,19 @@ void DebugWordChoices() {
 
   if (StopperDebugLevel >= 1 ||
     WordToDebug && BestChoices &&
-  StringSameAs (WordToDebug, (VIABLE_CHOICE) first (BestChoices))) {
+  StringSameAs (WordToDebug, WordToDebug_lengths,
+                (VIABLE_CHOICE) first_node (BestChoices))) {
     if (BestRawChoice)
-      PrintViableChoice (stdout, "\nBest Raw Choice:   ", BestRawChoice);
+      PrintViableChoice (stderr, "\nBest Raw Choice:   ", BestRawChoice);
 
     i = 1;
     Choices = BestChoices;
     if (Choices)
       cprintf ("\nBest Cooked Choices:\n");
-    iterate(Choices) { 
+    iterate(Choices) {
       sprintf (LabelString, "Cooked Choice #%d:  ", i);
-      PrintViableChoice (stdout, LabelString,
-        (VIABLE_CHOICE) first (Choices));
+      PrintViableChoice (stderr, LabelString,
+        (VIABLE_CHOICE) first_node (Choices));
       i++;
     }
   }
@@ -434,7 +458,7 @@ void DebugWordChoices() {
 
 
 /*---------------------------------------------------------------------------*/
-void FilterWordChoices() { 
+void FilterWordChoices() {
 /*
  **	Parameters: none
  **	Globals:
@@ -447,11 +471,11 @@ void FilterWordChoices() {
  */
   EXPANDED_CHOICE BestChoice;
 
-  if (BestChoices == NIL || second (BestChoices) == NIL)
+  if (BestChoices == NIL || second_node (BestChoices) == NIL)
     return;
 
   /* compute certainties and class for each chunk in best choice */
-  ExpandChoice ((VIABLE_CHOICE_STRUCT *) first (BestChoices), &BestChoice);
+  ExpandChoice ((VIABLE_CHOICE_STRUCT *) first_node (BestChoices), &BestChoice);
 
   set_rest (BestChoices, delete_d (rest (BestChoices),
     &BestChoice, FreeBadChoice));
@@ -499,8 +523,8 @@ FLOAT32 RatingMargin, FLOAT32 Thresholds[]) {
   assert (BestChoices != NIL);
   assert (BestRawChoice != NULL);
 
-  ExpandChoice(BestRawChoice, &BestRaw); 
-  Choice = (VIABLE_CHOICE) first (BestChoices);
+  ExpandChoice(BestRawChoice, &BestRaw);
+  Choice = (VIABLE_CHOICE) first_node (BestChoices);
 
   for (i = 0, Chunk = 0; i < Choice->Length; i++, Thresholds++) {
     AvgRating = 0.0;
@@ -528,7 +552,7 @@ FLOAT32 RatingMargin, FLOAT32 Thresholds[]) {
 
 
 /*---------------------------------------------------------------------------*/
-void InitStopperVars() { 
+void InitStopperVars() {
 /*
  **	Parameters: none
  **	Globals: none
@@ -541,20 +565,21 @@ void InitStopperVars() {
 
   string_variable (DangerousAmbigs, "DangerousAmbigs", DANGEROUS_AMBIGS);
   string_variable (WordToDebug, "WordToDebug", "");
+  string_variable (WordToDebug_lengths, "WordToDebug_lengths", "");
 
-  MakeNonDictCertainty(); 
-  MakeRejectCertaintyOffset(); 
-  MakeSmallWordSize(); 
-  MakeCertaintyPerChar(); 
-  MakeCertaintyVariation(); 
-  MakeStopperDebugLevel(); 
-  MakeAmbigThresholdGain(); 
-  MakeAmbigThresholdOffset(); 
+  MakeNonDictCertainty();
+  MakeRejectCertaintyOffset();
+  MakeSmallWordSize();
+  MakeCertaintyPerChar();
+  MakeCertaintyVariation();
+  MakeStopperDebugLevel();
+  MakeAmbigThresholdGain();
+  MakeAmbigThresholdOffset();
 }                                /* InitStopperVars */
 
 
 /*---------------------------------------------------------------------------*/
-void InitChoiceAccum() { 
+void InitChoiceAccum() {
 /*
  **	Parameters: none
  **	Globals: none
@@ -567,14 +592,14 @@ void InitChoiceAccum() {
   BLOB_WIDTH *BlobWidth, *End;
 
   if (BestRawChoice)
-    memfree(BestRawChoice); 
+    memfree(BestRawChoice);
 
   if (BestChoices)
-    destroy_nodes(BestChoices, memfree); 
+    destroy_nodes(BestChoices, memfree);
 
   BestRawChoice = NULL;
   BestChoices = NIL;
-  EnableChoiceAccum(); 
+  EnableChoiceAccum();
 
   for (BlobWidth = CurrentSegmentation,
     End = CurrentSegmentation + MAX_NUM_CHUNKS;
@@ -606,9 +631,9 @@ LogNewRawChoice (A_CHOICE * Choice, FLOAT32 AdjustFactor, float Certainties[]) {
     BestRawChoice = NewViableChoice (Choice, AdjustFactor, Certainties);
   else if (class_probability (Choice) < BestRawChoice->Rating) {
     if (ChoiceSameAs (Choice, BestRawChoice))
-      ReplaceDuplicateChoice(BestRawChoice, Choice, AdjustFactor, Certainties); 
+      ReplaceDuplicateChoice(BestRawChoice, Choice, AdjustFactor, Certainties);
     else {
-      memfree(BestRawChoice); 
+      memfree(BestRawChoice);
       BestRawChoice = NewViableChoice (Choice, AdjustFactor, Certainties);
     }
   }
@@ -616,7 +641,7 @@ LogNewRawChoice (A_CHOICE * Choice, FLOAT32 AdjustFactor, float Certainties[]) {
 
 
 /*---------------------------------------------------------------------------*/
-void LogNewSegmentation(PIECES_STATE BlobWidth) { 
+void LogNewSegmentation(PIECES_STATE BlobWidth) {
 /*
  **	Parameters:
  **		BlobWidth[]	number of chunks in each blob in segmentation
@@ -639,7 +664,7 @@ void LogNewSegmentation(PIECES_STATE BlobWidth) {
 
 
 /*---------------------------------------------------------------------------*/
-void LogNewSplit(int Blob) { 
+void LogNewSplit(int Blob) {
 /*
  **	Parameters:
  **		Blob	index of blob that was split
@@ -655,12 +680,12 @@ void LogNewSplit(int Blob) {
   LIST Choices;
 
   if (BestRawChoice) {
-    AddNewChunk(BestRawChoice, Blob); 
+    AddNewChunk(BestRawChoice, Blob);
   }
 
   Choices = BestChoices;
-  iterate(Choices) { 
-    AddNewChunk ((VIABLE_CHOICE) first (Choices), Blob);
+  iterate(Choices) {
+    AddNewChunk ((VIABLE_CHOICE) first_node (Choices), Blob);
   }
 
 }                                /* LogNewSplit */
@@ -705,16 +730,16 @@ FLOAT32 AdjustFactor, float Certainties[]) {
   /* see if a choice with the same text string has already been found */
   NewChoice = NULL;
   Choices = BestChoices;
-  iterate(Choices) { 
-    if (ChoiceSameAs (Choice, (VIABLE_CHOICE) first (Choices)))
+  iterate(Choices) {
+    if (ChoiceSameAs (Choice, (VIABLE_CHOICE) first_node (Choices)))
       if (class_probability (Choice) < BestRating (Choices))
-        NewChoice = (VIABLE_CHOICE) first (Choices);
+        NewChoice = (VIABLE_CHOICE) first_node (Choices);
     else
       return;
   }
 
   if (NewChoice) {
-    ReplaceDuplicateChoice(NewChoice, Choice, AdjustFactor, Certainties); 
+    ReplaceDuplicateChoice(NewChoice, Choice, AdjustFactor, Certainties);
     BestChoices = delete_d (BestChoices, NewChoice, is_same_node);
   }
   else {
@@ -723,22 +748,27 @@ FLOAT32 AdjustFactor, float Certainties[]) {
 
   BestChoices = s_adjoin (BestChoices, NewChoice, CmpChoiceRatings);
   if (StopperDebugLevel >= 2)
-    PrintViableChoice (stdout, "New Word Choice:  ", NewChoice);
+    PrintViableChoice (stderr, "New Word Choice:  ", NewChoice);
   if (count (BestChoices) > tessedit_truncate_wordchoice_log) {
     Choices =
       (LIST) nth_cell (BestChoices, tessedit_truncate_wordchoice_log);
     destroy_nodes (rest (Choices), Efree);
-    set_rest(Choices, NIL); 
+    set_rest(Choices, NIL);
   }
 
 }                                /* LogNewWordChoice */
 
 
 /*---------------------------------------------------------------------------*/
-int NoDangerousAmbig(char *Word, DANGERR *fixpt) { 
+static AMBIG_TABLE *AmbigFor = NULL;
+
+int NoDangerousAmbig(const char *Word,
+                     const char *Word_lengths,
+                     DANGERR *fixpt) {
 /*
  **	Parameters:
  **		Word	word to check for dangerous ambiguities
+ **		Word_lengths	lengths of unichars in Word
  **	Globals: none
  **	Operation: This word checks each letter in word against a list
  **		of potentially ambiguous characters.  If a match is found
@@ -751,9 +781,8 @@ int NoDangerousAmbig(char *Word, DANGERR *fixpt) {
  **	Exceptions: none
  **	History: Mon May  6 16:28:56 1991, DSJ, Created.
  */
-  static AMBIG_TABLE *AmbigFor = NULL;
 
-  char NewWord[MAX_WERD_SIZE];
+  char NewWord[MAX_WERD_SIZE * UNICHAR_LEN + 1];
   char *NextNewChar;
   int bad_index = 0;
 
@@ -762,13 +791,19 @@ int NoDangerousAmbig(char *Word, DANGERR *fixpt) {
 
   NextNewChar = NewWord;
   while (*Word)
-  if (AmbigsFound (NewWord, NextNewChar, Word + 1, AmbigFor[*Word], fixpt)) {
+  if (AmbigsFound (NewWord, NextNewChar,
+                   Word + *Word_lengths, Word_lengths + 1,
+                   AmbigFor[unicharset.unichar_to_id(Word, *Word_lengths)],
+                   fixpt)) {
     if (fixpt != NULL)
       fixpt->index = bad_index;
     return (FALSE);
   }
   else {
-    *NextNewChar++ = *Word++;
+    strncpy(NextNewChar, Word, *Word_lengths);
+    NextNewChar += *Word_lengths;
+    Word += *Word_lengths;
+    Word_lengths++;
     bad_index++;
   }
 
@@ -776,9 +811,18 @@ int NoDangerousAmbig(char *Word, DANGERR *fixpt) {
 
 }                                /* NoDangerousAmbig */
 
+void EndDangerousAmbigs() {
+  if (AmbigFor != NULL) {
+    for (int i = 0; i <= MAX_CLASS_ID; ++i) {
+      destroy_nodes(AmbigFor[i], Efree);
+    }
+    Efree(AmbigFor);
+    AmbigFor = NULL;
+  }
+}
 
 /*---------------------------------------------------------------------------*/
-void SettupStopperPass1() { 
+void SettupStopperPass1() {
 /*
  **	Parameters: none
  **	Globals:
@@ -794,7 +838,7 @@ void SettupStopperPass1() {
 
 
 /*---------------------------------------------------------------------------*/
-void SettupStopperPass2() { 
+void SettupStopperPass2() {
 /*
  **	Parameters: none
  **	Globals:
@@ -813,7 +857,7 @@ void SettupStopperPass2() {
               Private Code
 ----------------------------------------------------------------------------**/
 /*---------------------------------------------------------------------------*/
-void AddNewChunk(VIABLE_CHOICE Choice, int Blob) { 
+void AddNewChunk(VIABLE_CHOICE Choice, int Blob) {
 /*
  **	Parameters:
  **		Choice	choice to add a new chunk to
@@ -845,7 +889,8 @@ void AddNewChunk(VIABLE_CHOICE Choice, int Blob) {
 /*---------------------------------------------------------------------------*/
 int AmbigsFound(char *Word,
                 char *CurrentChar,
-                char *Tail,
+                const char *Tail,
+                const char *Tail_lengths,
                 LIST Ambigs,
                 DANGERR *fixpt) {
 /*
@@ -853,6 +898,7 @@ int AmbigsFound(char *Word,
  **		Word		word being tested for ambiguities
  **		CurrentChar	position in Word to put ambig replacement
  **		Tail		end of word to place after ambiguity
+ **		Tail_lengths    lengths of the unichars in Tail
  **		Ambigs		list of ambiguities to test at this position
  **	Globals: none
  **	Operation: For each ambiguity in Ambigs, see if the remainder of
@@ -867,37 +913,44 @@ int AmbigsFound(char *Word,
  **	Exceptions: none
  **	History: Thu May  9 10:10:28 1991, DSJ, Created.
  */
-  char *AmbigSpec;
-  char *UnmatchedTail;
+  AMBIG_SPEC *AmbigSpec;
+  char *ambig;
+  char *ambig_lengths;
+  const char *UnmatchedTail;
+  const char *UnmatchedTail_lengths;
   int Matches;
   int bad_length;
 
-  iterate(Ambigs) { 
-    AmbigSpec = (char *) first (Ambigs);
+  iterate(Ambigs) {
+    AmbigSpec = (AMBIG_SPEC *) first_node (Ambigs);
+    ambig = AmbigSpec->ambig;
+    ambig_lengths = AmbigSpec->lengths;
     bad_length = 1;
     UnmatchedTail = Tail;
+    UnmatchedTail_lengths = Tail_lengths;
     Matches = TRUE;
 
-    while (*AmbigSpec != ' ' && Matches)
-    if (*AmbigSpec == *UnmatchedTail) {
-      AmbigSpec++;
-      UnmatchedTail++;
-      bad_length++;
-    }
-    else
-      Matches = FALSE;
+    while (*ambig != ' ' && Matches)
+      if (*UnmatchedTail_lengths == *ambig_lengths &&
+          strncmp(ambig, UnmatchedTail, *ambig_lengths) == 0) {
+        ambig += *(ambig_lengths++);
+        UnmatchedTail += *(UnmatchedTail_lengths++);
+        bad_length++;
+      }
+      else
+        Matches = FALSE;
 
     if (Matches) {
-      AmbigSpec++;               /* skip over the space */
-                                 /* insert replacement string */
-      strcpy(CurrentChar, AmbigSpec); 
-                                 /* add tail */
-      strcat(Word, UnmatchedTail); 
+      ambig += *(ambig_lengths++); /* skip over the space */
+                                   /* insert replacement string */
+      strcpy(CurrentChar, ambig);
+                                   /* add tail */
+      strcat(Word, UnmatchedTail);
       if (valid_word (Word)) {
         if (StopperDebugLevel >= 1)
           cprintf ("Stopper:  Possible ambiguous word = %s\n", Word);
         if (fixpt != NULL) {
-          fixpt->good_length = strlen (AmbigSpec);
+          fixpt->good_length = strlen (ambig_lengths);
           fixpt->bad_length = bad_length;
         }
         return (TRUE);
@@ -910,7 +963,7 @@ int AmbigsFound(char *Word,
 
 
 /*---------------------------------------------------------------------------*/
-int ChoiceSameAs(A_CHOICE *Choice, VIABLE_CHOICE ViableChoice) { 
+int ChoiceSameAs(A_CHOICE *Choice, VIABLE_CHOICE ViableChoice) {
 /*
  **	Parameters:
  **		Choice		choice to compare to ViableChoice
@@ -923,7 +976,8 @@ int ChoiceSameAs(A_CHOICE *Choice, VIABLE_CHOICE ViableChoice) {
  **	Exceptions: none
  **	History: Fri May 17 08:48:04 1991, DSJ, Created.
  */
-  return (StringSameAs (class_string (Choice), ViableChoice));
+  return (StringSameAs (class_string (Choice), class_lengths (Choice),
+                        ViableChoice));
 
 }                                /* ChoiceSameAs */
 
@@ -957,7 +1011,7 @@ int CmpChoiceRatings(void *arg1,    //VIABLE_CHOICE                 Choice1,
 
 
 /*---------------------------------------------------------------------------*/
-void ExpandChoice(VIABLE_CHOICE Choice, EXPANDED_CHOICE *ExpandedChoice) { 
+void ExpandChoice(VIABLE_CHOICE Choice, EXPANDED_CHOICE *ExpandedChoice) {
 /*
  **	Parameters:
  **		Choice		choice to be expanded
@@ -984,7 +1038,7 @@ void ExpandChoice(VIABLE_CHOICE Choice, EXPANDED_CHOICE *ExpandedChoice) {
 
 
 /*---------------------------------------------------------------------------*/
-AMBIG_TABLE *FillAmbigTable() { 
+AMBIG_TABLE *FillAmbigTable() {
 /*
  **	Parameters: none
  **	Globals:
@@ -1007,36 +1061,65 @@ AMBIG_TABLE *FillAmbigTable() {
   FILE *AmbigFile;
   AMBIG_TABLE *NewTable;
   int i;
-  char TestString[256];
-  char ReplacementString[256];
-  char name[1024];
-  char *AmbigSpec;
-  int AmbigSize;
+  int AmbigPartSize;
+  char buffer[256 * UNICHAR_LEN];
+  char TestString[256 * UNICHAR_LEN];
+  char TestString_lengths[256];
+  char ReplacementString[256 * UNICHAR_LEN];
+  char ReplacementString_lengths[256];
+  STRING name;
+  char lengths[2];
+  AMBIG_SPEC *AmbigSpec;
+  UNICHAR_ID unichar_id;
 
-  strcpy(name, demodir); 
-  strcat(name, DangerousAmbigs); 
-  AmbigFile = Efopen (name, "r");
+  lengths[1] = 0;
+
+  name = language_data_path_prefix;
+  name += DangerousAmbigs;
+  AmbigFile = Efopen (name.string(), "r");
   NewTable = (AMBIG_TABLE *) Emalloc (sizeof (LIST) * (MAX_CLASS_ID + 1));
 
   for (i = 0; i <= MAX_CLASS_ID; i++)
     NewTable[i] = NIL;
 
-  while (fscanf (AmbigFile, "%s", TestString) == 1 &&
-  fscanf (AmbigFile, "%s", ReplacementString) == 1) {
-    if (strlen (TestString) > MAX_AMBIG_SIZE ||
-      strlen (ReplacementString) > MAX_AMBIG_SIZE)
+  while (fscanf (AmbigFile, "%d", &AmbigPartSize) == 1) {
+    TestString[0] = '\0';
+    TestString_lengths[0] = 0;
+    ReplacementString[0] = '\0';
+    ReplacementString_lengths[0] = 0;
+    for (i = 0; i < AmbigPartSize; ++i) {
+      fscanf (AmbigFile, "%s", buffer);
+      strcat(TestString, buffer);
+      lengths[0] = strlen(buffer);
+      strcat(TestString_lengths, lengths);
+    }
+    fscanf (AmbigFile, "%d", &AmbigPartSize);
+    for (i = 0; i < AmbigPartSize; ++i) {
+      fscanf (AmbigFile, "%s", buffer);
+      strcat(ReplacementString, buffer);
+      lengths[0] = strlen(buffer);
+      strcat(ReplacementString_lengths, lengths);
+    }
+
+    if (strlen (TestString_lengths) > MAX_AMBIG_SIZE ||
+        strlen (ReplacementString_lengths) > MAX_AMBIG_SIZE)
       DoError (0, "Illegal ambiguity specification!");
 
-    AmbigSize = strlen (TestString) + strlen (ReplacementString) + 1;
-    AmbigSpec = (char *) Emalloc (sizeof (char) * AmbigSize);
+    AmbigSpec = (AMBIG_SPEC *) Emalloc (sizeof (AMBIG_SPEC));
 
-    strcpy (AmbigSpec, &(TestString[1]));
-    strcat (AmbigSpec, " ");
-    strcat(AmbigSpec, ReplacementString); 
-    NewTable[TestString[0]] =
-      push_last (NewTable[TestString[0]], AmbigSpec);
+    strcpy(AmbigSpec->ambig, TestString + TestString_lengths[0]);
+    strcat(AmbigSpec->ambig, " ");
+    strcat(AmbigSpec->ambig, ReplacementString);
+
+    strcpy(AmbigSpec->lengths, TestString_lengths + 1);
+    lengths[0] = 1;
+    strcat(AmbigSpec->lengths, lengths);
+    strcat(AmbigSpec->lengths, ReplacementString_lengths);
+    unichar_id = unicharset.unichar_to_id(TestString, TestString_lengths[0]);
+    NewTable[unichar_id] = push_last (NewTable[unichar_id], AmbigSpec);
   }
-  fclose(AmbigFile); 
+
+  fclose(AmbigFile);
   return (NewTable);
 
 }                                /* FillAmbigTable */
@@ -1075,7 +1158,7 @@ int FreeBadChoice(void *item1,    //VIABLE_CHOICE                 Choice,
       if (Choice->Blob[i].Class != BestChoice->ChunkClass[Chunk] &&
     Choice->Blob[i].Certainty - BestChoice->ChunkCertainty[Chunk] <
       Threshold) {
-        memfree(Choice); 
+        memfree(Choice);
     return (TRUE);
   }
 
@@ -1085,10 +1168,11 @@ int FreeBadChoice(void *item1,    //VIABLE_CHOICE                 Choice,
 
 
 /*---------------------------------------------------------------------------*/
-int LengthOfShortestAlphaRun(register char *Word) { 
+int LengthOfShortestAlphaRun(register char *Word, const char *Word_lengths) {
 /*
  **	Parameters:
- **		Word	word to be tested
+ **		Word            word to be tested
+ **		Word_lengths    lengths of the unichars in Word
  **	Globals: none
  **	Operation: Return the length of the shortest alpha run in Word.
  **	Return:  Return the length of the shortest alpha run in Word.
@@ -1098,9 +1182,11 @@ int LengthOfShortestAlphaRun(register char *Word) {
   register int Shortest = MAXINT;
   register int Length;
 
-  for (; *Word; Word++)
-  if (isalpha (*Word)) {
-    for (Length = 1, Word++; isalpha (*Word); Word++, Length++);
+  for (; *Word; Word += *(Word_lengths++))
+  if (unicharset.get_isalpha(Word, *Word_lengths)) {
+    for (Length = 1, Word += *(Word_lengths++);
+         *Word && unicharset.get_isalpha(Word, *Word_lengths);
+         Word += *(Word_lengths++), Length++);
     if (Length < Shortest)
       Shortest = Length;
 
@@ -1135,10 +1221,11 @@ NewViableChoice (A_CHOICE * Choice, FLOAT32 AdjustFactor, float Certainties[]) {
   VIABLE_CHOICE NewChoice;
   int Length;
   char *Word;
+  char *Word_lengths;
   CHAR_CHOICE *NewChar;
   BLOB_WIDTH *BlobWidth;
 
-  Length = strlen (class_string (Choice));
+  Length = strlen (class_lengths (Choice));
   assert (Length <= MAX_NUM_CHUNKS && Length > 0);
 
   NewChoice = (VIABLE_CHOICE) Emalloc (sizeof (VIABLE_CHOICE_STRUCT) +
@@ -1148,12 +1235,13 @@ NewViableChoice (A_CHOICE * Choice, FLOAT32 AdjustFactor, float Certainties[]) {
   NewChoice->Certainty = class_certainty (Choice);
   NewChoice->AdjustFactor = AdjustFactor;
   NewChoice->Length = Length;
-
   for (Word = class_string (Choice),
-    NewChar = &(NewChoice->Blob[0]),
-    BlobWidth = CurrentSegmentation;
-  *Word; Word++, NewChar++, Certainties++, BlobWidth++) {
-    NewChar->Class = *Word;
+           Word_lengths = class_lengths (Choice),
+           NewChar = &(NewChoice->Blob[0]),
+           BlobWidth = CurrentSegmentation;
+       *Word;
+       Word += *(Word_lengths++), NewChar++, Certainties++, BlobWidth++) {
+    NewChar->Class = unicharset.unichar_to_id(Word, *Word_lengths);
     NewChar->NumChunks = *BlobWidth;
     NewChar->Certainty = *Certainties;
   }
@@ -1164,7 +1252,7 @@ NewViableChoice (A_CHOICE * Choice, FLOAT32 AdjustFactor, float Certainties[]) {
 
 
 /*---------------------------------------------------------------------------*/
-void PrintViableChoice(FILE *File, const char *Label, VIABLE_CHOICE Choice) { 
+void PrintViableChoice(FILE *File, const char *Label, VIABLE_CHOICE Choice) {
 /*
  **	Parameters:
  **		File	open text file to print Choice to
@@ -1185,11 +1273,11 @@ void PrintViableChoice(FILE *File, const char *Label, VIABLE_CHOICE Choice) {
     Choice->Rating, Choice->Certainty, Choice->AdjustFactor);
 
   for (i = 0; i < Choice->Length; i++)
-    fprintf (File, "%c", Choice->Blob[i].Class);
+    fprintf (File, "%s", unicharset.id_to_unichar(Choice->Blob[i].Class));
   fprintf (File, "\n");
 
   for (i = 0; i < Choice->Length; i++) {
-    fprintf (File, "  %c", Choice->Blob[i].Class);
+    fprintf (File, "  %s", unicharset.id_to_unichar(Choice->Blob[i].Class));
     for (j = 0; j < Choice->Blob[i].NumChunks - 1; j++)
       fprintf (File, "   ");
   }
@@ -1228,6 +1316,7 @@ FLOAT32 AdjustFactor, float Certainties[]) {
  **	History: Fri May 17 13:35:58 1991, DSJ, Created.
  */
   char *Word;
+  char *Word_lengths;
   CHAR_CHOICE *NewChar;
   BLOB_WIDTH *BlobWidth;
 
@@ -1236,9 +1325,11 @@ FLOAT32 AdjustFactor, float Certainties[]) {
   OldChoice->AdjustFactor = AdjustFactor;
 
   for (Word = class_string (NewChoice),
-    NewChar = &(OldChoice->Blob[0]),
-    BlobWidth = CurrentSegmentation;
-  *Word; Word++, NewChar++, Certainties++, BlobWidth++) {
+           Word_lengths = class_lengths (NewChoice),
+           NewChar = &(OldChoice->Blob[0]),
+           BlobWidth = CurrentSegmentation;
+       *Word;
+       Word += *(Word_lengths++), NewChar++, Certainties++, BlobWidth++) {
     NewChar->NumChunks = *BlobWidth;
     NewChar->Certainty = *Certainties;
   }
@@ -1246,10 +1337,13 @@ FLOAT32 AdjustFactor, float Certainties[]) {
 
 
 /*---------------------------------------------------------------------------*/
-int StringSameAs(const char *String, VIABLE_CHOICE ViableChoice) { 
+int StringSameAs(const char *String,
+                 const char *String_lengths,
+                 VIABLE_CHOICE ViableChoice) {
 /*
  **	Parameters:
  **		String		string to compare to ViableChoice
+ **		String_lengths	lengths of unichars in String
  **		ViableChoice	viable choice to compare to String
  **	Globals: none
  **	Operation: This routine compares String to ViableChoice and
@@ -1260,11 +1354,17 @@ int StringSameAs(const char *String, VIABLE_CHOICE ViableChoice) {
  */
   CHAR_CHOICE *Char;
   int i;
+  int current_unichar_length;
 
   for (Char = &(ViableChoice->Blob[0]), i = 0;
-    i < ViableChoice->Length; String++, Char++, i++)
-  if (*String != Char->Class)
+    i < ViableChoice->Length;
+       String += *(String_lengths++), Char++, i++) {
+    current_unichar_length = strlen(unicharset.id_to_unichar(Char->Class));
+  if (current_unichar_length != *String_lengths ||
+      strncmp(String, unicharset.id_to_unichar(Char->Class),
+              current_unichar_length) != 0)
     return (FALSE);
+  }
 
   if (*String == 0)
     return (TRUE);
@@ -1275,7 +1375,7 @@ int StringSameAs(const char *String, VIABLE_CHOICE ViableChoice) {
 
 
 /*---------------------------------------------------------------------------*/
-int UniformCertainties(CHOICES_LIST Choices, A_CHOICE *BestChoice) { 
+int UniformCertainties(CHOICES_LIST Choices, A_CHOICE *BestChoice) {
 /*
  **	Parameters:
  **		Choices		choices for current segmentation
@@ -1310,7 +1410,7 @@ int UniformCertainties(CHOICES_LIST Choices, A_CHOICE *BestChoice) {
     return (TRUE);
 
   TotalCertainty = TotalCertaintySquared = 0.0;
-  for_each_choice(Choices, i) { 
+  for_each_choice(Choices, i) {
     CharChoices = (CHOICES) array_index (Choices, i);
     Certainty = best_certainty (CharChoices);
     TotalCertainty += Certainty;
